# -*- coding: utf-8 -*-
#*******************************************************************************
#* © 2009-2017, Cypress Semiconductor Corporation
#* or a subsidiary of Cypress Semiconductor Corporation. All rights
#* reserved.
#* 
#* This software, including source code, documentation and related
#* materials (“Software”), is owned by Cypress Semiconductor
#* Corporation or one of its subsidiaries (“Cypress”) and is protected by
#* and subject to worldwide patent protection (United States and foreign),
#* United States copyright laws and international treaty provisions.
#* Therefore, you may use this Software only as provided in the license
#* agreement accompanying the software package from which you
#* obtained this Software (“EULA”).
#* 
#* If no EULA applies, Cypress hereby grants you a personal, non-
#* exclusive, non-transferable license to copy, modify, and compile the
#* Software source code solely for use in connection with Cypress’s
#* integrated circuit products. Any reproduction, modification, translation,
#* compilation, or representation of this Software except as specified
#* above is prohibited without the express written permission of Cypress.
#* 
#* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO
#* WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING,
#* BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
#* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#* PARTICULAR PURPOSE. Cypress reserves the right to make
#* changes to the Software without notice. Cypress does not assume any
#* liability arising out of the application or use of the Software or any
#* product or circuit described in the Software. Cypress does not
#* authorize its products for use in any products where a malfunction or
#* failure of the Cypress product may reasonably be expected to result in
#* significant property damage, injury or death (“High Risk Product”). By
#* including Cypress’s product in a High Risk Product, the manufacturer
#* of such system or application assumes all risk of such use and in doing
#* so agrees to indemnify Cypress against all liability.
#********************************************************************************
import win32com.client
import array
import PPCOM
from PPCOM import enumInterfaces
from PPCOM import enumFrequencies
from PPCOM import enumSonosArrays

#Define global variables
m_sLastError = ""

#Distinguishing identifier of PSoC3/5 families
LEOPARD_ID = 0xE0;
PANTHER_ID = 0xE1;

def SUCCEEDED(hr):
    return hr >= 0

def GetGenerationByJtagID(hr):
    return hr >= 0

#Check JTAG ID of device - identify family PSoC3 or PSoC5
def GetGenerationByJtagID(JtagID):
    distinguisher = (((ord(JtagID[0]) & 0x0F) << 4) | (ord(JtagID[1]) >> 4))
    return distinguisher

def IsPSoC3ES3(jtagID):
    global LEOPARD_ID
    if (GetGenerationByJtagID(jtagID) == LEOPARD_ID):
        if ((ord(jtagID[0]) >> 4) >= 0x01): return 1  #silicon version==0x01 saved in bits [4..7]
                                                 #For ES0-2==0x00, ES3==0x01
    return 0

def Is_PSoC5_TM_ID(jtagID):    
    return ((ord(jtagID[0]) & 0xFF) == 0x0E) and ((ord(jtagID[1]) & 0xF0) == 0x10) and (ord(jtagID[3]) == 0x69)

def Is_PSoC_5_LP_ID(jtagID):
    #Check whether SWD ID belongs to PSoC5LP
    #PSoC5LP: 0x1BA01477 (SWD/JTAG read ID read request retuns CM3 ID always)
    #         0x2E1xx069 (LP) - this Device ID must be read from PANTHER_DEVICE_ID reg (0x4008001C)
    #2E0xx069
    return (((ord(jtagID[0]) & 0xFF) >= 0x2E) and ((ord(jtagID[1]) & 0xF0) == 0x10) and (ord(jtagID[3]) == 0x69))

def Allow_Reset_Hard():
    hResult = pp.DAP_GetJtagID();
    hr = hResult[0]
    chipJtagID = hResult[1]
    if (Is_PSoC_5_LP_ID(chipJtagID) == True):
        #Set 'allow_rst_hrd' bit in MLOGIC_DEBUG register (see PSoC5 IROS: 001-43078, Book 2)
        hr = pp.DAP_WriteIO(0x400046E8, 0x00000040);
    return hr

def OpenPort():
    global m_sLastError
    # Open Port - get last (connected) port in the ports list
    hResult = pp.GetPorts()
    hr = hResult[0]
    portArray = hResult[1]
    m_sLastError = hResult[2]    
    if (not SUCCEEDED(hr)): return hr
    if (len(portArray) <= 0):
        m_sLastError = "Connect any Programmer to PC"
        return -1
    bFound = 0
    for i in range(0, len(portArray)):
        if (portArray[i].startswith("MiniProg3") or portArray[i].startswith("DVKProg") or portArray[i].startswith("FirstTouch") or portArray[i].startswith("Gen-FX2LP") or portArray[i].startswith("KitProg")):
            portName = portArray[i]            
            bFound = 1;
            break
    if(bFound == 0):
        m_sLastError = "Connect any MiniProg3/DVKProg/FirstTouch/Gen-FX2LP device to the PC"
        return -1

    #Port should be opened just once to connect Programmer device (MiniProg1/3,etc).
    #After that you can use Chip-/Programmer- specific APIs as long as you need.
    #No need to repoen port when you need to acquire chip 2nd time, just call Acquire() again.
    #This is true for all other APIs which get available once port is opened.
    #You have to call OpenPort() again if port was closed by ClosePort() method, or
    #when there is a need to connect to other programmer, or
    #if programmer was physically reconnected to USB-port.
            
    hr = pp.OpenPort(portName)
    m_sLastError = hr[1]
    return hr[0]

def ClosePort():    
    hResult = pp.ClosePort()
    hr = hResult[0]
    strError = hResult[1]
    return hr

def CheckHexAndDeviceCompatibility():
    global m_sLastError
    listResult = []
    result = 0
    hResult = pp.DAP_GetJtagID();
    hr = hResult[0]
    chipJtagID = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)):
        listResult.append(hr)
        listResult.append(result)
        return listResult
    hResult = pp.HEX_ReadJtagID();
    hr = hResult[0]
    hexJtagID = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)):
        listResult.append(hr)
        listResult.append(result)
        return listResult
    result = 1
    for i in range(0, 4):
        if(ord(hexJtagID[i]) != ord(chipJtagID[i])):
            result = 0
            break
    listResult.append(0)
    listResult.append(result)
    return listResult

def IsNvlUpdatePossible():
    global m_sLastError
    listResult = []
    hResult = pp.GetPower1()
    hr = hResult[0]
    power = hResult[1]
    voltage = hResult[2]
    state = 0
    if (not SUCCEEDED(hr)):
        listResult.append(hr)
        listResult.append(0)
        return listResult
    if (voltage > 3700):        
        state = 0
    else:
        state = 1
    listResult.append(hr)
    listResult.append(state)
    return listResult

def ProgramNvlArrays(nvlArrayType):
    global m_sLastError
    hResult = pp.PSoC3_GetSonosArrays(nvlArrayType)   
    hr = hResult[0]
    arrayInfo = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr
    addrHex = 0
    for i in range(0, len(arrayInfo[0])):
        arrayID = arrayInfo[0][i]
        arraySize = arrayInfo[1][i]
        #Read data from Hex file
        if (nvlArrayType == enumSonosArrays.ARRAY_NVL_USER):
            hResult = pp.PSoC3_ReadHexNvlCustom(addrHex, arraySize)
        else: #enumSonosArrays.ARRAY_NVL_WO_LATCHES
            hResult = pp.PSoC3_ReadHexNvlWo(addrHex, arraySize)
        hr = hResult[0]
        hexData = hResult[1]
        m_sLastError = hResult[2]
        if (not SUCCEEDED(hr)): return hr        
        addrHex += arraySize
        #Read data from device
        hResult = pp.PSoC3_ReadNvlArray(arrayID)
        hr = hResult[0]
        chipData = hResult[1]
        m_sLastError = hResult[2]
        if (not SUCCEEDED(hr)): return hr
        #Compare data from Chip against corresponding Hex-block
        if (len(chipData) != len(hexData)):
            m_sLastError = "Hex file's NVL array differs from corresponding device's one!"
            return -1
        fIdentical = 1
        for i in range(0, arraySize):
            if (ord(chipData[i]) != ord(hexData[i])):
                fIdentical = 0
                break
        if (fIdentical == 1): continue #Arrays are equal, skip programming, goto following array
        #Verify voltage range for TM silicon. Do not allow TM programming if voltage is greater than 3.3 V
        hResult = pp.DAP_GetJtagID()
        hr = hResult[0]
        chipJtagID = hResult[1]
        m_sLastError = hResult[2]
        if (not SUCCEEDED(hr)): return hr
        if (Is_PSoC5_TM_ID(chipJtagID)):
            hResult = IsNvlUpdatePossible()
            hr = hResult[0]
            state = hResult[1]
            if (state == 0):                
                if(nvlArrayType == enumSonosArrays.ARRAY_NVL_USER):
                    m_sLastError = "User NVLs"
                else:
                    m_sLastError = "WO NVLs"
                m_sLastError = m_sLastError + " update failed. This operation can be completed in voltage range 1.8 - 3.3 V only."
                return -1
        #Program NVL array
        hResult = pp.PSoC3_WriteNvlArray(arrayID, hexData);
        hr = hResult[0]
        m_sLastError = hResult[1]
        if (not SUCCEEDED(hr)): return hr

        #PSoC3 ES3 support - check whether ECCEnable bit is changed and reacquire device
        if (nvlArrayType == enumSonosArrays.ARRAY_NVL_USER):
            hResult = pp.DAP_GetJtagID()
            hr = hResult[0]
            chipJtagID = hResult[1]
            m_sLastError = hResult[2]
            if (not SUCCEEDED(hr)): return hr
            #ES3 and probably newer revisions
            if (IsPSoC3ES3(chipJtagID) or Is_PSoC_5_LP_ID(chipJtagID)):
                eccEnableChanged = 0
                if (len(hexData) >= 4):
                    eccEnableChanged = ((ord(hexData[3]) ^ ord(chipData[3])) & 0x08) == 0x08
                #need to reacquire chip if EccEnable bit was changed to apply it for flash rows.
                if (eccEnableChanged):
                    hResult = pp.DAP_Acquire()
                    hr = hResult[0]
                    m_sLastError = hResult[1]
            return hr
    return 0

def GetEccOption(arrayID):
    global m_sLastError
    hResult = pp.PSoC3_GetFlashArrayInfo(arrayID)
    hr = hResult[0]
    rowSize = hResult[1]
    rowsPerArray = hResult[2]
    eccPresence = hResult[3]
    m_sLastError = hResult[4]
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.PSoC3_GetEccStatus() #get ecc status of the acquired device
    hr = hResult[0]
    eccHwStatus = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr
    #take into account data from the Config area
    if ((eccPresence != 0) and (eccHwStatus == 0)):
        eccOption = 1
    else:
        eccOption = 0    
    return eccOption

def ProgramFlashArrays(flashSize):
    global m_sLastError
    hResult = pp.PSoC3_GetSonosArrays(enumSonosArrays.ARRAY_FLASH)
    hr = hResult[0]
    arrayInfo = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr

    flashProgrammed = 0
    #Program Flash arrays
    for i in range(0, len(arrayInfo[0])):
        arrayID = arrayInfo[0][i]
        arraySize = arrayInfo[1][i]
        #Program flash array from hex file taking into account ECC (Config Data)                
        hResult = pp.PSoC3_GetFlashArrayInfo(arrayID)
        hr = hResult[0]
        rowSize = hResult[1]
        rowsPerArray = hResult[2]
        eccPresence = hResult[3]
        m_sLastError = hResult[4]
        if (not SUCCEEDED(hr)): return hr
        eccOption = GetEccOption(arrayID); #take into account data from the Config area
        for rowID in range(0, rowsPerArray):
            hResult = pp.PSoC3_ProgramRowFromHex(arrayID, rowID, eccOption)
            hr = hResult[0]
            m_sLastError = hResult[1]
            if (not SUCCEEDED(hr)): return hr
            #Limit programming to the device flash size
            flashProgrammed += rowSize
            if (flashProgrammed >= flashSize): return hr
    return hr

def GetTotalFlashRowsCount(flashSize):
    global m_sLastError
    listResult = []
    totalRows = 0
    rowSize = 256
    totalRows = flashSize / rowSize
    listResult.append(0)
    listResult.append(totalRows)
    return listResult

def CheckSum_All(flashSize):
    global m_sLastError
    listResult = []
    cs = 0
    hResult = pp.PSoC3_GetSonosArrays(enumSonosArrays.ARRAY_FLASH)
    hr = hResult[0]
    arrayInfo = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)):
        listResult.append(hr)
        listResult.append(cs)
        return listResult
    hResult = GetTotalFlashRowsCount(flashSize)
    hr = hResult[0]
    RowsToChecksum = hResult[1]    
    for i in range(0, len(arrayInfo[0])):
        arrayID = arrayInfo[0][i]
        arraySize = arrayInfo[1][i]
        #Get info about flash array
        hResult = pp.PSoC3_GetFlashArrayInfo(arrayID)
        hr = hResult[0]
        rowSize = hResult[1]
        rowsPerArray = hResult[2]
        eccPresence = hResult[3]
        m_sLastError = hResult[4]
        if (not SUCCEEDED(hr)):
            listResult.append(hr)
            listResult.append(cs)
            return listResult
        #Find number of rows in array to be checksumed
        if ((RowsToChecksum - rowsPerArray) < 0):
            rowsPerArray = RowsToChecksum
        #Calculate checksum of the array
        arrayChecksum = 0;
        hResult = pp.PSoC3_CheckSum(arrayID, 0, rowsPerArray)
        hr = hResult[0]
        arrayChecksum = hResult[1]
        m_sLastError = hResult[2]
        if (not SUCCEEDED(hr)):
            listResult.append(hr)
            listResult.append(cs)
            return listResult
        #Sum checksum
        cs += arrayChecksum
        #Update number of rows to be checksumed
        RowsToChecksum -= rowsPerArray
        if (RowsToChecksum <= 0):
            break;
    listResult.append(hr)
    listResult.append(cs)
    return listResult

def ProgramAll():
    global m_sLastError
    #Setup Power - "2.5V" and internal
    hResult = pp.SetPowerVoltage("2.5")
    hr = hResult[0]
    m_sLastError = hResult[1]    
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.PowerOn()
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr    
    #Set protocol, connector and frequency
    hResult = pp.SetProtocol(enumInterfaces.SWD)
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.SetProtocolConnector(1); #10-pin connector
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr    
    hResult = pp.SetProtocolClock(enumFrequencies.FREQ_03_0); #3.0 MHz clock on SWD bus
    hr = hResult[0]
    m_sLastError = hResult[1]
    # Set Hex File
    hResult = pp.ReadHexFile("C:\\Users\\MBRakshith\\Desktop\\Test\\Design01.hex")
    hr = hResult[0]
    hexImageSize = int(hResult[1])
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr
    # Set Acquire Mode
    pp.SetAcquireMode("Reset")

    #The "Programming Flow" proper
    #Acquire Device
    hResult = pp.DAP_Acquire()
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    #Check Hex File and Device compatibility
    fCompatibility = 0
    hResult = CheckHexAndDeviceCompatibility()    
    hr = hResult[0]
    fCompatibility = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    if (fCompatibility == 0):
        m_sLastError = "The Hex file does not match the acquired device, please connect the appropriate device";
        return -1
    #Erase All
    hResult = pp.PSoC3_EraseAll();
    hr = hResult[0]
    m_sLastError = hResult[1]    
    if (not SUCCEEDED(hr)): return hr

    #Program User NVL arrays
    hr = ProgramNvlArrays(enumSonosArrays.ARRAY_NVL_USER)
    if (not SUCCEEDED(hr)): return hr
    #Program Flash arrays
    hr = ProgramFlashArrays(hexImageSize)
    #Protect All arrays
    hResult = pp.PSoC3_ProtectAll()
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr

    #CheckSum
    hResult = CheckSum_All(hexImageSize)
    hr = hResult[0]
    flashChecksum = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.ReadHexChecksum()
    hr = hResult[0]
    hexChecksum = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr
    flashChecksum &= 0xFFFF
    if (flashChecksum != hexChecksum):
        print "Mismatch of Checksum: Expected 0x%x, Got 0x%x" %(flashChecksum, hexChecksum)
        return -1
    else:
        print "Checksum 0x%x" %(flashChecksum)
    #Release PSoC3 device
    #For PSoC5 LP call "Allow_Hard_Reset()" as in C#/C++ example
    hResult = Allow_Reset_Hard()
    hResult = pp.DAP_ReleaseChip()
    hr = hResult[0]
    m_sLastError = hResult[1]
    return hr

def UpgradeBlock():
    global m_sLastError
    #Setup Power - "2.5V" and internal
    hResult = pp.SetPowerVoltage("2.5")
    hr = hResult[0]
    m_sLastError = hResult[1]    
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.PowerOn()
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr    
    #Set protocol, connector and frequency
    hResult = pp.SetProtocol(enumInterfaces.SWD)
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    hResult = pp.SetProtocolConnector(1); #10-pin connector
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr    
    hResult = pp.SetProtocolClock(enumFrequencies.FREQ_03_0); #<=3.0 MHz for Read operations
    hr = hResult[0]
    m_sLastError = hResult[1]

    #Set Acquire Mode
    pp.SetAcquireMode("Reset")

    #Acquire Device
    hResult = pp.DAP_Acquire()
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr    

    #Write Row, use PSoC3_WriteRow() instead PSoC3_ProgramRow()
    writeData = [] #User and Config area of the Row (256+32)    
    for i in range(0, 288):
        writeData.append(i & 0xFF)
    data = array.array('B',writeData)
    arrayID = 0
    rowID = 255    
    hResult = pp.PSoC3_WriteRow(arrayID, rowID, buffer(data))
    hr = hResult[0]
    m_sLastError = hResult[1]
    if (not SUCCEEDED(hr)): return hr
    
    #Verify Row - only user area (without Config one)
    hResult = pp.PSoC3_ReadRow(arrayID, rowID, 0)    
    hr = hResult[0]
    readRow = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr

    for i in range(0, len(readRow)): #check 256 bytes
        if (ord(readRow[i]) != writeData[i]):
            hr = -1
            break
    if (not SUCCEEDED(hr)):
        m_sLastError = "Verification of User area failed!"
        return hr

    #Verify Row - Config are only
    hResult = pp.PSoC3_ReadRow(arrayID, rowID, 1)
    hr = hResult[0]
    readRow = hResult[1]
    m_sLastError = hResult[2]
    if (not SUCCEEDED(hr)): return hr

    for i in range(0, len(readRow)): #check 256 bytes
        if (ord(readRow[i]) != writeData[256+i]):
            hr = -1
            break
    if (not SUCCEEDED(hr)):
        m_sLastError = "Verification of Config area failed!"
        return hr

    #Release PSoC3 chip
    #For PSoC5 LP call "Allow_Hard_Reset()" as in C#/C++ example
    hResult = pp.DAP_ReleaseChip()
    hr = hResult[0]
    m_sLastError = hResult[1]
    return hr
        
def Execute():    
    hr = OpenPort()
    if (not SUCCEEDED(hr)): return hr
    hr = ProgramAll()
    #hr = UpgradeBlock()
    #pp.DAP_AcquireChip()
    #pp.DAP_ReleaseChip()
    ClosePort()
    return hr

#Begin main program

#Use Version Independent Prog ID to instantiate COM-object
pp = win32com.client.Dispatch("PSoCProgrammerCOM.PSoCProgrammerCOM_Object")
#For version dependent Prog ID use below commented line, but update there COM-object version (e.g. 14)
#pp = win32com.client.Dispatch("PSoCProgrammerCOM.PSoCProgrammerCOM_Object.14")

print "Program All using COM-object interface only"
hr = Execute()
if (SUCCEEDED(hr)):
    str = "Succeeded!"
else:
    str = "Failed! " + m_sLastError
print str
raw_input("Press any key to exit")
#End main function
